var fs       = require("fs");
var mustache = require("mustache");

var _template_cache = {};

// This code is from node_mailer It loads a template from a file, given by the |message|'s
// |template| property.  After the template is loaded, calls callback with the contents
// processed by Mustache.
// Object -> (Object -> undefined) -> undefined
function afterLoadingTemplate(message, callback) {
    if (_template_cache[message.template]) {
        if (_template_cache[message.template].loaded) {
            if (_template_cache[message.template].template) {
                message.body = mustache.to_html(_template_cache[message.template].template, message.data);
            } else {
                message.text = mustache.to_html(_template_cache[message.template].text, message.data);
                message.html = mustache.to_html(_template_cache[message.template].html, message.data);
            }
            callback(null, message);
        } else {
            _template_cache[message.template].queue.push(message);
        }
    } else {
        _template_cache[message.template]          = {};
        _template_cache[message.template].loaded   = false;
        _template_cache[message.template].template = '';
        _template_cache[message.template].queue    = [];

        fs.readFile(message.template, function(err, result){
            if (err) {
                // assume that the file does not exit

                return fs.readFile(message.template + ".text.mustache", function (err, text) {
                    if (err) return callback(err); // TODO: do not give up
                    return fs.readFile(message.template + ".html.mustache", function (err, html) {
                        if (err) return callback(err); // TODO: do not give up

                        _template_cache[message.template].text = text.toString();
                        _template_cache[message.template].html = html.toString();
                        _template_cache[message.template].loaded   = true;

                        // "Drain" the queue
                        _template_cache[message.template].queue.push(message);
                        _template_cache[message.template].queue.forEach(function(msg, i){
                            msg.text = mustache.to_html(_template_cache[message.template].text, msg.data);
                            msg.html = mustache.to_html(_template_cache[message.template].html, msg.data);
                            callback(null, msg);
                        });

                        // Clear the queue out
                        _template_cache[message.template].queue = [];
                    });
                });
            }

            _template_cache[message.template].template = result.toString();
            _template_cache[message.template].loaded   = true;

            // "Drain" the queue
            _template_cache[message.template].queue.push(message);
            _template_cache[message.template].queue.forEach(function(msg, i){
                msg.body = mustache.to_html(_template_cache[message.template].template, msg.data);
                callback(null, msg);
            });

            // Clear the queue out
            _template_cache[message.template].queue = [];
        });
    }
}

module.exports = {
    afterLoadingTemplate : afterLoadingTemplate
};
